
public class Shooter implements Runnable {
	
	// flag used to stop the program
	public static boolean isRunning = true;
	
	// the four friends
	private Friend friend1;
	private Friend friend2;
	private Friend friend3;
	private Friend friend4;
	
	// stores max number of zombies allowed in the room
	private final int N;
	
	// constructor takes in the max number of zombies allowed in the room (originally from command line).
	public Shooter(int max){
		N = max;
	}
	
	// adds friends to the gunner
	public void addFriends(Friend a, Friend b, Friend c, Friend d){
		friend1 = a;
		friend2 = b;
		friend3 = c;
		friend4 = d;
	}
	
	// radios to each friend, one-by-one to sum up the total number of zombies that have been let in since the start
	public int getCurrentTotal(){
		return friend1.getCount() + friend2.getCount() + friend3.getCount() + friend4.getCount();
	}

	
	public void run(){
		
		// count of how many zombies were successfully killed
		int zombiesShot = 0;
		
		// number of zombies killed for a particular second
		int throughPut = 0;
		
		// total number of zombies that are known to have been let in according to the last check
		int currentTotalLetIn = 0;
		
		// number of zombies currently in the room.  assume that it starts with one zombie in the room
		// 	so that the gunner doesnt wait 2 seconds before starting to shoot.
		int currentTotalInRoom = 1;
		
		// total number of zombies killed during the previous check
		int oldCount = 0;
		
		// number of seconds elapsed (lower bound - ie: at least this many second has passed)
		int secondsElapsed = 0;
		
		// create threads for friends
		Thread alice = new Thread(friend1);
		Thread bob = new Thread(friend2);
		Thread carl = new Thread(friend3);
		Thread dianne = new Thread(friend4);
		
		// start threads for friends
		alice.start();
		bob.start();
		carl.start();
		dianne.start();
		
		// cotinuously perform the same actions:
        while(isRunning) {
        	// every time the 10ms sleep occurs 200 times, it's been (at least) 2 seconds
        	for (int i = 0; i < 200; i++){
	        	try {
	    			if(Math.random() < 0.40 && currentTotalInRoom>0){
	    				// there's a 40% chance
	    				// kill zombie if there are any zombies to kill
	    				if(currentTotalInRoom >0 ){
	    					zombiesShot++;	// increment kill count
	    					currentTotalInRoom --;	// adjust number of zombies in room
	    				}
	    				//System.out.println("Hit!  "+ currentTotalInRoom +" zombies in the room");
	    			} else {
	    				if (currentTotalInRoom <= 0){
	    					//System.out.println("Hold Fire! "+ currentTotalInRoom +" zombies in the room");
	    				} else {
	    					//System.out.println("Miss! "+ currentTotalInRoom +" zombies in the room");
	    				}
	    			}
	    			
	    			// every two seconds perform the check
	        		if(i == 199){
	        			// check other threads
	        			currentTotalLetIn = getCurrentTotal();
	        			currentTotalInRoom = currentTotalLetIn - zombiesShot;
	        			
	        			if (currentTotalInRoom > N){
	        				//System.out.println("more than N zombies in the room: "+ currentTotalInRoom);
	        				friend1.setPermission(false);
	        				friend2.setPermission(false);
	        				friend3.setPermission(false);
	        				friend4.setPermission(false);
	        				
	        			} else if (currentTotalInRoom < N/2){
	        				//System.out.println("less than N/2 zombies in the room: "+ currentTotalInRoom);
	        				friend1.setPermission(true);
	        				friend2.setPermission(true);
	        				friend3.setPermission(true);
	        				friend4.setPermission(true);
	        			} else {
	        				//System.out.println("less than N, but more than N/2 zombies in the room: " + currentTotalInRoom);
	        			}
	        		}
	        		//every second consider throughput
	        		if (i == 99 || i == 199){
	        			
        				secondsElapsed++;	// increment total number of seconds elapsed
	        			throughPut +=  (zombiesShot-oldCount);	// current throughput = count-oldCount
	        			
	        			System.out.println("Current Throughput: "+(zombiesShot-oldCount)+".  Average throughput: "+throughPut/secondsElapsed+" zombies killed per second.  It's been (at least) "+secondsElapsed+"s.");
	        			oldCount = zombiesShot;	// make oldCount = to current count because next time current count will be the old count
	        		}

	                Thread.sleep(10);	// sleep for 10ms
	            } catch(InterruptedException ie) {
	            	
	            }
        	}
        	// way of telling threads when to stop.
    		if (secondsElapsed >=200){
    			isRunning = false;	// stops the program after 200s
    		}
        }
	}
}